/**
 * AP_Logger.h
 *
 * Apama logging API declarations for use only in the IAF and IAF plugins (not correlator plugins).
 *
 * $Copyright(c) 2002-2009, 2011-2013 Progress Software Corporation (PSC). All rights reserved.$
 * $Copyright (c) 2013-2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 * $Id: AP_Logger.h 340061 2018-10-29 10:52:53Z jhea $
 */

#ifndef AP_LOGGER_H
#define AP_LOGGER_H


#include <AP_Platform.h>
#include <stdarg.h>

/** \file
 * Logging Support.
 */

/**
 * Enumeration of logging verbosity levels.  In order of increasing verbosity,
 * the levels are: NULL < OFF < FORCE < CRIT < FATAL < ERROR < WARN < INFO < DEBUG.  Messages logged
 * at level X will be sent to the log if the current level is >= X.
 */
enum AP_LogLevel {
	AP_LogLevel_NULL,
	AP_LogLevel_OFF,
	AP_LogLevel_FORCE,
	AP_LogLevel_CRIT,
	AP_LogLevel_FATAL,
	AP_LogLevel_ERROR,
	AP_LogLevel_WARN,
	AP_LogLevel_INFO,
	AP_LogLevel_DEBUG,
	AP_LogLevel_TRACE
};


#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

	/**
	 * Log a message at TRACE level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_TRACE or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogTrace() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogTrace(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at DEBUG level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_DEBUG or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogDebug() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogDebug(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at INFO level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_INFO or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogInfo() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogInfo(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at WARN level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_WARN or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogWarn() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogWarn(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at ERROR level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_ERROR or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogError() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogError(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Always log a message
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogCrit() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogForce(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at CRIT level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_CRIT or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogCrit() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogCrit(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at FATAL level.  Note that the message will only appear
	 * in the log file if the current logging level is AP_LogLevel_FATAL or
	 * greater.
	 *
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_LogFatal() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_LogFatal(const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 1, 2)))
#endif	
	;

	/**
	 * Log a message at a specific log level.
	 *
	 * @param level The level to log the message at.  The message will only
	 * appear in the log file if the current logging level is equal to or
	 * greater than the specified level.
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * in the args parameter.
	 * @param args Standard C varargs argument list holding any additional
	 * data required by the printf format string in message.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_DoLog(enum AP_LogLevel level, const char* message, va_list args)
#ifdef __GNUC__
		__attribute__ ((format (printf, 2, 0)))
#endif	
	;

	/**
	 * Log a message at a specific log level.
	 *
	 * @param level The level to log the message at.  The message will only
	 * appear in the log file if the current logging level is equal to or
	 * greater than the specified level.
	 * @param message The message to be logged.  This is a standard printf()
	 * format string; any values required by the formatting should be passed
	 * as additional arguments to the AP_DoLogV() call.
	 */
	AP_COMMON_API void AP_COMMON_CALL AP_DoLogV(enum AP_LogLevel level, const char* message, ...)
#ifdef __GNUC__
		__attribute__ ((format (printf, 2, 3)))
#endif	
	;

	/**
	 * Utility function to convert a character string to a logging verbosity
	 * level from the AP_LogLevel enumeration.  May be useful if you wish to
	 * set the log level based on the value of a configuration parameter.
	 *
	 * @param str The string to be converted.  A case-insensitive comparison
	 * is performed against the values "OFF", "CRIT", "FATAL", "ERROR", "WARN", "INFO",
	 * "DEBUG" and "TRACE".
	 * @return The logging level corresponding to str.  AP_LogLevel_NULL will
	 * be returned for any unrecognised string.
	 */
	AP_COMMON_API enum AP_LogLevel AP_COMMON_CALL AP_StringToLogLevel(const char* str);

	/**
	 * Utility function to convert a logging verbosity level from the
	 * AP_LogLevel enumeration to a human-readable character string.
	 *
	 * @param level The logging level to be converted.
	 * @return A character string corresponding to the logging level.  This
	 * will be one of the strings "OFF", "CRIT", "FATAL", "ERROR", "WARN", "INFO", "DEBUG",
	 * "TRACE" or "NULL".  The string "NULL" will be returned for any unknown level.
	 */
	AP_COMMON_API const char* AP_COMMON_CALL AP_LogLevelToString(enum AP_LogLevel level);

	/**
	 * Get the current logging level.
	 *
	 * @return The current logging level
	 */
	AP_COMMON_API enum AP_LogLevel AP_COMMON_CALL AP_GetLogLevel();

	/**
	 * Get the current logging destination.
	 *
	 * @return The current logging file name.  This may be one of the special
	 * values "stderr" or "stdout".  In all cases, the returned string is
	 * owned by the logger and should not be freed or otherwise modified by
	 * the caller.
	 */
	AP_COMMON_API const char* AP_COMMON_CALL AP_GetLogFileName();

	AP_COMMON_API int AP_COMMON_CALL AP_GetLogFD();

	/**
	 * Get the current logging destination.
	 *
	 * @return The current logging file namein UTF8.  This may be one of the special
	 * values "stderr" or "stdout".  In all cases, the returned string is
	 * owned by the logger and should not be freed or otherwise modified by
	 * the caller.
	 */
	AP_COMMON_API const char* AP_COMMON_CALL AP_GetUTF8LogFileName();


#ifdef __cplusplus
}  /* extern "C" */
#endif  /* __cplusplus */


#endif  /* AP_LOGGER_H */
