/*
 * NormalisedEvent.h
 *
 * External (client-visible) interface to the normalised event type used to
 * pass event into and out of the IAF Semantic Mapper component.  Normalised
 * events are essentially just dictionaries of name-value pairs, where both
 * the names and values are character strings.  Each name-value pair nominally
 * represents the name and content of a single field from an event, but users
 * of the class are free to invent custom naming schemes to represent more
 * complex event structures.  Names must be unique within a given event.
 * Values may be empty or missing.
 *
 * Please note that normalised events are NOT thread-safe.  If your code will
 * be acessing the same normalised event object (or associated iterators) from
 * multiple threads, you must implement your own thread synchronisation to
 * prevent concurrent modification of the objects.
 *
 * $Copyright(c) 2002-2006, 2008 Progress Software Corporation (PSC). All rights reserved.$
 * $Copyright (c) 2013-2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 * $Revision: 340684 $
 */

#ifndef AP_NORMALISED_EVENT_H
#define AP_NORMALISED_EVENT_H


#include <AP_Types.h>
#include <IAF_Platform.h>

/** \file 
 * Normalised Event.
 * External (client-visible) interface to the normalised event type used to
 * pass event into and out of the IAF Semantic Mapper component.  Normalised
 * events are essentially just dictionaries of name-value pairs, where both
 * the names and values are character strings.  Each name-value pair nominally
 * represents the name and content of a single field from an event, but users
 * of the class are free to invent custom naming schemes to represent more
 * complex event structures.  Names must be unique within a given event.
 * Values may be empty or missing.
 *
 * Please note that normalised events are NOT thread-safe.  If your code will
 * be acessing the same normalised event object (or associated iterators) from
 * multiple threads, you must implement your own thread synchronisation to
 * prevent concurrent modification of the objects.
 */

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */


/*
 * Forward declare some classes
 */
struct AP_NormalisedEvent;
struct AP_NormalisedEventIterator;


/**
 * Event field names (keys) are NUL-terminated C character strings.
 */
typedef const AP_char8* AP_NormalisedEventKey;


/**
 * Event field values are NUL-terminated C character strings.
 */
typedef const AP_char8* AP_NormalisedEventValue;


/**
 * AP_NormalisedEventIterator_Functions
 *
 * Table of client visible functions exported by a normalised event iterator
 * (an object used for stepping through the contents of a normalised event in
 * forward or reverse order).  These functions declare the only operations
 * that may be performed by users of the normalised event.
 * decoding-specific functions).
 *
 * Note that all of these functions take an initial
 * AP_NormalisedEventIterator* argument; this is analogous to the (hidden)
 * 'this' pointer passed to a C++ object when a member function is invoked on
 * it.
 */
struct AP_NormalisedEventIterator_Functions {

	/**
	 * valid
	 *
	 * Check whether the iterator points to a valid element of the normalised
	 * event.  Typically used as part of the loop condition when iterating
	 * over the contents of an event.
	 *
	 * @param it The event iterator instance. 
	 * @return AP_TRUE if the iterator points to a valid element, AP_FALSE
	 * otherwise.
	 */
	AP_bool (*valid)(struct AP_NormalisedEventIterator* it);

	/**
	 * key
	 *
	 * Return the key (name) associated with the current event element pointed
	 * to by the iterator.  The returned value is owned by the underlying
	 * normalised event and should not be modified or freed by the caller.
	 *
	 * @param it The event iterator instance. 
	 * @return The name of the current normalised event element
	 */
	AP_NormalisedEventKey (*key)(struct AP_NormalisedEventIterator* it);

	/**
	 * value
	 *
	 * Return the value associated with the current event element pointed to
	 * by the iterator.  The returned value is owned by the underlying
	 * normalised event and should not be modified or freed by the caller.
	 * The returned value may be an empty string "", or NULL to indicate a
	 * missing value.
	 *
	 * @param it The event iterator instance. 
	 * @return The value of the current normalised event element
	 */
	AP_NormalisedEventValue (*value)(struct AP_NormalisedEventIterator* it);

	/**
	 * next
	 *
	 * Move the iterator to the next element of the underlying normalised
	 * event instance.  The iterator must be in a valid state (such that the
	 * valid() function would return AP_TRUE) before this function is called.
	 *
	 * Note that the order in which elements is returned is not necessarily
	 * the same as the order in which they were added.  The order may change
	 * as elements are added to or removed from the underlying event.
	 *
	 * @param it The event iterator instance. 
	 * @return AP_TRUE if the iterator still points to a valid event element
	 * after moving forwards, AP_FALSE otherwise.
	 */
	AP_bool (*next)(struct AP_NormalisedEventIterator* it);

	/**
	 * back
	 * Move the iterator to the previous element of the underlying normalised
	 * event instance.  The iterator must be in a valid state (such that the
	 * valid() function would return AP_TRUE) before this function is called.
	 *
	 * Note that the order in which elements is returned is not necessarily
	 * the same as the order in which they were added.  The order may change
	 * as elements are added to or removed from the underlying event.
	 *
	 * @param it The event iterator instance. 
	 * @return AP_TRUE if the iterator still points to a valid event element
	 * after moving backwards, AP_FALSE otherwise.
	 */
	AP_bool (*back)(struct AP_NormalisedEventIterator* it);
};


/**
 * AP_NormalisedEventIterator
 *
 * External (client-visible) interface to a normalised event iterator object.
 * The AP_NormalisedEventIterator struct contains a table of function
 * pointers, declared in the AP_NormalisedEventIterator_Functions struct
 * above.  The implementation of these functions is private.  Users of the
 * iterator should invoke functions on it as in the following example
 * (iterator is of type AP_NormalisedEventIterator*):
 *
 * b = iterator->functions->valid(iterator);
 *
 * Note that an iterator is only valid as long as the underlying normalised
 * event remains unchanged.  Any iterators on a given event should be deleted
 * as soon as any operation is performed that might change the contents of the
 * event object.
 */
struct AP_NormalisedEventIterator {

	/**
	 * Pointer to private internal data.
	 */
	void* reserved;

	/**
	 * Function table of iterator operations.  See documentation for
	 * AP_NormalisedEventIterator_Functions for details.
	 */
	struct AP_NormalisedEventIterator_Functions* functions;

};
typedef struct AP_NormalisedEventIterator AP_NormalisedEventIterator;


/**
 * AP_NormalisedEventIterator_dtor
 *
 * Destroys a normalised event iterator object.  This function should be
 * called as soon as the iterator is no longer required, to free any resources
 * it occupies.
 *
 * @param it The iterator to be destroyed.
 */
AP_IAFCORE_API void AP_IAFCORE_CALL AP_NormalisedEventIterator_dtor(AP_NormalisedEventIterator* it);


/**
 * AP_NormalisedEvent_Functions
 *
 * Table of client visible functions exported by a normalised event instance.
 * These are the only operations that may be performed by users of the event
 * object.
 *
 * Note that all of these functions take an initial AP_NormalisedEvent*
 * argument; this is analogous to the (hidden) 'this' pointer passed to a C++
 * object when a member function is invoked on it.
 *
 * For all functions that return a NormalisedEventValue pointer, the string
 * pointed to by that pointer is valid until the NormalisedEvent is destroyed
 * or an operation that changes the NormalisedEvent is called.
 */
struct AP_NormalisedEvent_Functions {

	/**
	 * size
	 *
	 * Get the number of elements (name-value pairs) currently stored by the
	 * event.
	 *
	 * @param event The normalised event instance
	 * @return The number of elements in the event
	 */
	AP_uint32 (*size)(struct AP_NormalisedEvent* event);

	/**
	 * empty
	 *
	 * Check whether the event is empty or not
	 *
	 * @param event The normalised event instance
	 * @return AP_TRUE if the event has zero elements, AP_FALSE otherwise
	 */
	AP_bool (*empty)(struct AP_NormalisedEvent* event);

	/**
	 * add
	 *
	 * Add a new name-value pair to the event.  The given name and value will
	 * be copied into the event.  If an element with the same name already
	 * exists, it will NOT be overwritten.
	 *
	 * @param event The normalised event instance
	 * @param key The name of the new element
	 * @param value the value of the new element
	 * @return An iterator pointing to the new element, or NULL if the element
	 * was not added (typically because the name already existed). Must be 
	 * explicitly deleted by the caller after use. 
	 */
	AP_NormalisedEventIterator* (*add)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key, AP_NormalisedEventValue value);

	/**
	 * addQuick
	 *
	 * Add a new name-value pair to the event.  The given name and value will
	 * be copied into the event.  If an element with the same name already
	 * exists, it will NOT be overwritten.
	 *
	 * This function is faster and more memory-efficient than add(), since it
	 * does not bother creating an iterator.  However, you have no way of
	 * knowing whether the element was actually added or not, other than
	 * searching for the name and checking the stored value.
	 *
	 * @param event The normalised event instance
	 * @param key The name of the new element
	 * @param value The value of the new element
	 */
	void (*addQuick)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key, AP_NormalisedEventValue value);

	/**
	 * remove
	 *
	 * Remove the named element from the normalised event.
	 *
	 * @param event The normalised event instance
	 * @param key The name of the element to be removed
	 */
	void (*remove)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key);

	/**
	 * replace
	 *
	 * Change the value of a named element in the normalised event.  If an
	 * element with the given name already exists, its value will be replaced
	 * with a copy of the given value.  Otherwise, a new element is created
	 * just as though addQuick() had been called.
	 *
	 * @param event The normalisedEvent instance
	 * @param key The name of the element to be updated
	 * @param value The new value for the named element
	 */
	void (*replace)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key, AP_NormalisedEventValue newValue);

	/**
	 * removeAll
	 *
	 * Remove all elements from the normalised event.  The empty() function
	 * will return AP_TRUE after this function has been called.
	 *
	 * @param event The normalised event instance
	 */
	void (*removeAll)(struct AP_NormalisedEvent* event);

	/**
	 * exists
	 *
	 * Check whether a given element exists in the normalised event.
	 *
	 * @param event The normalised event instance
	 * @param key The element name to search for
	 * @return AP_TRUE if an element with the given name exists in the
	 * normalised event, AP_FALSE otherwise.
	 */
	AP_bool (*exists)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key);

	/**
	 * find
	 *
	 * Search for a named element in the normalised event.
	 *
	 * @param event The normalised event instance
	 * @param key The element name to search for
	 * @return An iterator pointing to the element with the given name, or
	 * NULL if no matching element could be found. Must be explicitly 
	 * deleted by the caller after use. 
	 */
	AP_NormalisedEventIterator* (*find)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key);

	/**
	 * findValue
	 *
	 * Search for a named element in the normalised event and return its
	 * value.
	 *
	 * @param event The normalised event instance
	 * @param key The element name to search for
	 * @return The value associated with the given name, or NULL if no
	 * matching element could be found.  Note that this function cannot
	 * distinguish between an element with a missing value and an element that
	 * does not exist - NULL will be returned in either case.
	 */
	AP_NormalisedEventValue (*findValue)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key);

	/**
	 * findValueAndRemove
	 *
	 * Search for a named element in the normalised event and return its
	 * value.  If found, the element will also be removed from the event.
	 *
	 * @param event The normalised event instance
	 * @param key The element name to search for
	 * @return The value associated with the given name, or NULL if no
	 * matching element could be found.  If the element was found, it will be
	 * deleted from the event.  Note that this function cannot distinguish
	 * between an element with a missing value and an element that does not
	 * exist - NULL will be returned in either case. The returned object 
	 * (if any) must be explicitly deleted by the caller after use. 
	 */
	AP_NormalisedEventValue (*findValueAndRemove)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key);

	/**
	 * findValueAndRemove2
	 *
	 * Search for a named element in the normalised event and return its
	 * value.  If found, the element will also be removed from the event.
	 *
	 * @param event The normalised event instance
	 * @param key The element name to search for
	 * @param existed Out parameter that will be set to AP_TRUE if the named
	 * element was found and deleted.
	 * @return The value associated with the given name, or NULL if no
	 * matching element could be found.  If the element was found, it will be
	 * deleted from the event. The returned object 
	 * (if any) must be explicitly deleted by the caller after use. 
	 */
	AP_NormalisedEventValue (*findValueAndRemove2)(struct AP_NormalisedEvent* event, AP_NormalisedEventKey key, AP_bool* existed);  

	/**
	 * first
	 *
	 * Return an iterator pointing to the first element of the normalised
	 * event.  Successive calls to the next() function of the returned
	 * iterator will allow you to visit all the elements of the event.
	 *
	 * @param event The normalised event instance
	 * @return An iterator pointing to the first element of the event, or NULL
	 * if the event has no elements. Must be explicitly deleted by 
	 * the caller after use. 
	 */
	AP_NormalisedEventIterator* (*first)(struct AP_NormalisedEvent* event);

	/**
	 * last
	 *
	 * Return an iterator pointing to the last element of the normalised
	 * event.  Successive calls to the back() function of the returned
	 * iterator will allow you to visit all the elements of the event.
	 *
	 * @param event The normalised event instance
	 * @return An iterator pointing to the last element of the event, or NULL
	 * if the event has no elements. Must be explicitly deleted by 
	 * the caller after use. 
	 */
	AP_NormalisedEventIterator* (*last)(struct AP_NormalisedEvent* event);

	/**
	 * toString
	 *
	 * Return a printable string representation of the normalised event.  The
	 * returned string is owned by the caller and should be freed when it is
	 * no longer required using the char8free function.
	 *
	 * @param event The normalised event instance
	 * @return Printable string representation of the event
	 */
	AP_char8* (*toString)(struct AP_NormalisedEvent* event);
	
	
	/**
	 * char8free
	 *
	 * Free a dynamically-allocated AP_char8* string, such as that returned
	 * by the toString() function.  It should NOT be used to free arbitrary 
	 * objects allocated by user code.
	 *
	 * @param string The object to be freed
	 */
	void (*char8free)(AP_char8* string);	

};


/**
 * AP_NormalisedEvent
 *
 * External (client-visible) interface to a normalised event object.  The
 * AP_NormalisedEvent struct contains a table of function pointers, declared
 * in the AP_NormalisedEvent_Functions struct above.  The implementation of
 * these functions is private.  Users of the event should invoke functions on
 * it as in the following example (event is of type AP_NormalisedEvent*):
 *
 * it = event->functions->first(event);
 */
struct AP_NormalisedEvent {

	/**
	 * Pointer to private internal data.
	 */
	void* reserved;

	/**
	 * Function table of internal operations on the event object.  See
	 * documentation of AP_NormalisedEvent_Functions for details.
	 */
	struct AP_NormalisedEvent_Functions* functions;

};
typedef struct AP_NormalisedEvent AP_NormalisedEvent;


/*
 * AP_NormalisedEvent_globalInit
 *
 * Initialisation function used internally by the IAF.  User code should *not*
 * call this function under any circumstances.
 */
AP_IAFCORE_API void AP_IAFCORE_CALL AP_NormalisedEvent_globalInit();


/**
 * AP_NormalisedEvent_ctor
 *
 * Constructs a new normalised event instance.
 *
 * @return Pointer to a new normalised event instance, or NULL if the new
 * instance could not be created for some reason.
 */
AP_IAFCORE_API AP_NormalisedEvent* AP_IAFCORE_CALL AP_NormalisedEvent_ctor();


/**
 * AP_NormalisedEvent_copy
 *
 * Creates a deep copy of an existing normalised event instance.
 *
 * @param old The existing normalised event to be copied.
 * @return Pointer to a new normalised event instance whose contents are an
 * exact copy of old, or NULL if the new instance could not be created for
 * some reason.
 */
AP_IAFCORE_API AP_NormalisedEvent* AP_IAFCORE_CALL AP_NormalisedEvent_copy(AP_NormalisedEvent* old);


/**
 * AP_NormalisedEvent_dtor
 *
 * Destroys a normalised event object.  This function should be called as soon
 * as the event is no longer required, to free any resources it occupies.
 *
 * @param event The event to be destroyed.
 */
AP_IAFCORE_API void AP_IAFCORE_CALL AP_NormalisedEvent_dtor(AP_NormalisedEvent* event);


#ifdef __cplusplus
}  /* extern "C" */
#endif  /* __cplusplus */


#endif  /* AP_NORMALISED_EVENT_H */
